import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_FILEBROWSER_REMOVE = "mythic_rpc_filebrowser_remove"

class MythicRPCRemoveFileData:
    def __init__(self,
                 Host: str = None,
                 Path: str = None,
                 **kwargs):
        self.Host = Host
        self.Path = Path
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "host": self.Host,
            "path": self.Path
        }


class MythicRPCFileBrowserRemoveMessage:
    def __init__(self,
                 TaskID: int,
                 RemovedFiles: list[MythicRPCRemoveFileData],
                 **kwargs):
        self.TaskID = TaskID
        self.RemovedFiles = RemovedFiles
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "removed_files": [x.to_json() for x in self.RemovedFiles]
        }


class MythicRPCFileBrowserRemoveMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileBrowserRemove(
        msg: MythicRPCFileBrowserRemoveMessage) -> MythicRPCFileBrowserRemoveMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILEBROWSER_REMOVE,
                                                                            body=msg.to_json())
    return MythicRPCFileBrowserRemoveMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_KEYLOG_SEARCH = "mythic_rpc_keylog_search"

class MythicRPCKeylogSearchData:
    def __init__(self,
                 WindowTitle: str = None,
                 User: bool = None,
                 Keystrokes: dict = None,
                 **kwargs):
        self.WindowTitle = WindowTitle
        self.User = User
        self.Keystrokes = Keystrokes
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "window_title": self.WindowTitle,
            "user": self.User,
            "keystrokes": self.Keystrokes,
        }

class MythicRPCKeylogSearchResultData:
    def __init__(self,
                 window_title: str = None,
                 user: bool = None,
                 keystrokes: dict = None,
                 **kwargs):
        self.WindowTitle = window_title
        self.User = user
        self.Keystrokes = keystrokes
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "window_title": self.WindowTitle,
            "user": self.User,
            "keystrokes": self.Keystrokes,
        }


class MythicRPCKeylogSearchMessage:
    def __init__(self,
                 TaskID: int,
                 Keylog: MythicRPCKeylogSearchData,
                 **kwargs):
        self.TaskID = TaskID
        self.Keylog = Keylog
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "keylogs": self.Keylog.to_json()
        }


class MythicRPCKeylogSearchMessageResponse:
    Keylogs: list[MythicRPCKeylogSearchResultData]

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 keylogs: list[dict] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Keylogs = [MythicRPCKeylogSearchResultData(**x) for x in keylogs] if keylogs is not None else []
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCKeylogCreate(
        msg: MythicRPCKeylogSearchMessage) -> MythicRPCKeylogSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_KEYLOG_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCKeylogSearchMessageResponse(**response)



import aiohttp
from .config import settings
from .logging import logger


async def sendFileToMythic(contents: bytes, agentFileId: str) -> bool:
    try:
        async with aiohttp.ClientSession() as session:
            data = aiohttp.FormData()
            url = f"http://{settings.get('mythic_server_host')}:{settings.get('mythic_server_port', 17443)}/direct/upload/{agentFileId}"
            data.add_field('file', contents, filename='payload', )
            async with session.post(url, data=data, ssl=False) as resp:
                if resp.status == 200:
                    responseData = await resp.json()
                    if "status" in responseData and responseData["status"] == "success":
                        logger.info(f"[+] Successfully uploaded file contents to Mythic")
                        return True
                    else:
                        logger.error(f"[-] Failed to upload file contents to Mythic")
                        return False
    except Exception as e:
        logger.exception(f"[-] Failed to upload payload contents: {e}")
        return False


async def getFileFromMythic(agentFileId: str) -> bytes:
    try:
        async with aiohttp.ClientSession() as session:
            url = f"http://{settings.get('mythic_server_host')}:{settings.get('mythic_server_port', 17443)}/direct/download/{agentFileId}"
            async with session.get(url, ssl=False) as resp:
                if resp.status == 200:
                    responseData = await resp.read()
                    return responseData
    except Exception as e:
        logger.exception(f"[-] Failed to upload payload contents: {e}")
        return None


import mythic_container
from mythic_container.logging import logger
from mythic_container.utils_mythic_file_transfer import getFileFromMythic

MYTHIC_RPC_FILE_GET_CONTENT = "mythic_rpc_file_get_content"


class MythicRPCFileGetContentMessage:
    def __init__(self,
                 AgentFileId: str,
                 **kwargs):
        self.AgentFileId = AgentFileId
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "file_id": self.AgentFileId,
        }


class MythicRPCFileGetContentMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 content: bytes = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Content = content
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileGetContent(
        msg: MythicRPCFileGetContentMessage) -> MythicRPCFileGetContentMessageResponse:
    content = await getFileFromMythic(agentFileId=msg.AgentFileId)

    return MythicRPCFileGetContentMessageResponse(
        success=content is not None,
        error="Failed to fetch file from Mythic" if content is None else "",
        content=content
    )



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_FILE_REGISTER = "mythic_rpc_file_register"


class MythicRPCFileRegisterMessage:
    def __init__(self,
                 OperationId: int,
                 OperatorId: int = None,
                 DeleteAfterFetch: bool = False,
                 Filename: str = None,
                 Comment: str = "",
                 **kwargs):
        self.DeleteAfterFetch = DeleteAfterFetch
        self.Filename = Filename
        self.Comment = Comment
        self.OperationId = OperationId
        self.OperatorId = OperatorId
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "delete_after_fetch": self.DeleteAfterFetch,
            "filename": self.Filename,
            "comment": self.Comment,
            "operation_id": self.OperationId,
            "operator_id": self.OperatorId
        }


class MythicRPCFileRegisterMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 agent_file_id: str = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.AgentFileId = agent_file_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileRegister(
        msg: MythicRPCFileRegisterMessage) -> MythicRPCFileRegisterMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILE_REGISTER,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCFileRegisterMessageResponse(**response)
    return finalResponse



import mythic_container
from mythic_container.logging import logger
import base64

MYTHIC_RPC_AGENTSTORAGE_SEARCH = "mythic_rpc_agentstorage_search"


class MythicRPCAgentStorageSearchMessage:
    def __init__(self,
                 SearchUniqueID: str,
                 **kwargs):
        self.SearchUniqueID = SearchUniqueID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "unique_id": self.SearchUniqueID
        }


class MythicRPCAgentStorageSearchResult:
    def __init__(self,
                 unique_id: str = "",
                 data: str = "",
                 **kwargs):
        self.UniqueID = unique_id
        self.Data = base64.b64decode(data)
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


class MythicRPCAgentStorageSearchMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 agentstorage_messages: [MythicRPCAgentStorageSearchResult] = [],
                 **kwargs):
        self.Success = success
        self.Error = error
        self.AgentStorageMessages = agentstorage_messages
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCAgentStorageSearch(
        msg: MythicRPCAgentStorageSearchMessage) -> MythicRPCAgentStorageSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_AGENTSTORAGE_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCAgentStorageSearchMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_ARTIFACT_SEARCH = "mythic_rpc_artifact_search"


class MythicRPCArtifactSearchArtifactData:
    def __init__(self,
                 host: str = None,
                 artifact_type: str = None,
                 artifact_message: str = None,
                 task_id: int = None,
                 **kwargs):
        self.Host = host
        self.ArtifactType = artifact_type
        self.ArtifactMessage = artifact_message
        self.TaskID = task_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "host": self.Host,
            "artifact_type": self.ArtifactType,
            "artifact_message": self.ArtifactMessage,
            "task_id": self.TaskID
        }


class MythicRPCArtifactSearchMessage:
    def __init__(self,
                 TaskID: int,
                 SearchArtifacts: MythicRPCArtifactSearchArtifactData,
                 **kwargs):
        self.TaskID = TaskID
        self.SearchArtifacts = SearchArtifacts
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "artifact": self.SearchArtifacts.to_json()
        }


class MythicRPCArtifactSearchMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 artifacts: [dict] = [],
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Artifacts = [MythicRPCArtifactSearchArtifactData(**x) for x in artifacts]
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCArtifactSearch(
        msg: MythicRPCArtifactSearchMessage) -> MythicRPCArtifactSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_ARTIFACT_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCArtifactSearchMessageResponse(**response)
